/*
 * This software is copyright (c) 2004 by Shadowplay.net. It is released 
 * under the terms of the Artistic License.  Please refer to the file 
 * LICENSE for details.
 */
package net.shadowplay.http.cookie;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.shadowplay.wurml.WurML;


/**
 * @author Michael Dykman for Shadowplay.net 
 */
public class CookieJar
{
	private Map map;
	int hash;
	public CookieJar()
	{
		map = new HashMap();
		hash = 0;
	}
	public void load(String f)
		throws IOException
	{
		File file = new File(f);
		if(file.exists())
		{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String line;
			while((line = reader.readLine()) != null)
			{
				int offset = line.indexOf('=');
				String name = line.substring(0,offset);
				String value = line.substring(offset+1);
				addCookie(new Cookie(name,value),false);
			}
		}
		else
		{
			WurML.logNL("cookie file `" + f + "' not found, starting with no cookies");
		}
		calcHash();
	}
	public void save(String f)
		throws IOException
	{
		PrintWriter out = new PrintWriter(
			new FileOutputStream(f));
		Iterator it = map.values().iterator();
		while(it.hasNext())
		{
			Cookie cc = (Cookie) it.next();
			out.println(cc.name + "=" + cc.value);
		}
		out.close();
	}
	public void addCookie(Cookie c)
	{
		addCookie(c,true);
	}
	
	public void addCookie(Cookie c, boolean calc)
	{
		map.put(c.name,c);
		if(calc)
		{
			calcHash();
		}
	}
	
	public void addCookies(Map headers, boolean show)
	{
		Iterator it = headers.keySet().iterator();
		String key;
		Cookie c;
		while(it.hasNext())
		{
			key = (String) it.next();
			if(key != null && key.equalsIgnoreCase("set-cookie"))
			{
				List vals = (List) headers.get(key);
				Iterator lit = vals.iterator();
				while(lit.hasNext())
				{
					c = Cookie.makeCookie((String)lit.next(),show);
					if(c != null)
					{
						addCookie(c,false);
					}
				}
			}
		}
		calcHash();
	}
	
	public String cookieString()
	{
		Cookie cookie;
		StringBuffer sb = new StringBuffer();

		if(map.size() > 0)
		{
			Iterator it = map.values().iterator();
			while(it.hasNext())
			{
				cookie = (Cookie) it.next();
				if(sb.length() != 0)
				{
					sb.append(' ');
				}
				sb.append(cookie.name).append('=').append(cookie.value).append(';');
			}
			return sb.toString();
		}
		else
		{
			return null;
		}
		
	}
	
	public int hashCode()
	{
//System.out.println("CookieJar hash = " + hash);		
		return hash;
	}
	private void calcHash()
	{
		hash = 0;
		Iterator it = map.keySet().iterator();
		while(it.hasNext())
		{
			Object key = it.next();
			hash ^= key.hashCode();
			Object value = map.get(key);
			if(value != null)
			{
				hash ^= value.hashCode();
			}
		}
	}

}
