/*
 * This software is copyright (c) 2004 by Shadowplay.net. It is released 
 * under the terms of the Artistic License.  Please refer to the file 
 * LICENSE for details.
 */
package net.shadowplay.http.robot;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;


/**
 * @author Michael Dykman for Shadowplay.net 
 */
public class RobotDb
{
	private static final Integer noRobot = new Integer(-1);
	private Map hostBots;
	private String botAgent;
	private String userAgent;
	public RobotDb(String botAgent, String userAgent)
	{
		this.botAgent = botAgent;
		this.userAgent = userAgent;
		hostBots= new HashMap();
	}
	public synchronized boolean allow(URL base)
		throws IOException
	{
		boolean allow = true;
		String host = base.getHost();
		Object o = hostBots.get(host);
		if(o == null)
		{
			URL bot = new URL(base,"/robots.txt");
			HttpURLConnection conn = (HttpURLConnection) bot.openConnection();
			conn.setRequestProperty("User-Agent",userAgent);
			int rc = conn.getResponseCode();
			if(rc == HttpURLConnection.HTTP_OK)
			{
				RobotEntry r = new RobotEntry(botAgent);
				r.load(conn.getInputStream());
				o = r;
			}
			else
			{
				o = noRobot;
			}
			hostBots.put(host,o);
		}
		
		if(! o.equals(noRobot))
		{
			
			RobotEntry bot = (RobotEntry) o;
			String file = base.getFile();
			if(file == null || file.length() == 0)
			{
				file = "/";
			}
			allow = bot.allowed(file);
		}
		
		return allow;
	}
}
