/*
 * This software is copyright (c) 2004 by Shadowplay.net. It is released 
 * under the terms of the Artistic License.  Please refer to the file 
 * LICENSE for details.
 */
package net.shadowplay.http.robot;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * @author Michael Dykman for Shadowplay.net 
 */
public class RobotEntry
{
	private String agent;
	private ArrayList disallows;
	
	public RobotEntry(String agent)
	{
		this.agent = agent;
		this.disallows = new ArrayList();
	}
	public boolean allowed(String resource)
	{
		boolean allow = true;
		Iterator it = disallows.iterator();
		while(it.hasNext())
		{
			if(resource.startsWith(it.next().toString()))
			{
				allow = false;
				break;
			}
		}
		
		return allow;
	}
	public void load(InputStream is)
		throws IOException
	{
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		String line;
		boolean interested = false;
		while((line = reader.readLine()) != null)
		{
			if(line.startsWith("User-agent"))
			{
				int offset = line.indexOf(':');
				if(offset != -1 && line.length() > offset + 1)
				{
					String a = line.substring(offset+1).trim();
					if(a.equals("*") || a.equalsIgnoreCase(agent))
					{
						interested = true;
					}
					else
					{
						interested = false;
					}
				}
			}
			else if(interested && line.startsWith("Disallow"))
			{
				int offset = line.indexOf(':');
				if(offset != -1 && line.length() > offset + 1)
				{
					String a = line.substring(offset+1).trim();
					disallows.add(a);
				}
			}
		}
		disallows.trimToSize();
	}
}
