package net.shadowplay.wurml;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;

class XMLInputStream extends InputStream
{
	private static int READ_BUFFER_SIZE = 2096;
	private static final int MAX_HEADER_LEN = 160;
	private InputStream in;
	private int count;
	boolean echo = true;
	private ByteArrayOutputStream buffer;
//	private String encoding;
	byte[] bytes = null;
	private Logger logger;
	
//	private static byte[] ENCODING_8_BIT = new byte[] { 
//		'e', 'n', 'c', 'o', 'd', 'i', 'n', 'g' };
	private static byte[] ENCODING_16_BIT = new byte[]	{ 
		'e', 0, 'n', 0, 'c', 0, 'o', 0, 'd', 0, 'i', 0, 'n', 0, 'g', 0 };
//	private static byte[] ENCODING_16_BE = new byte[]	{ 
//		0, 'e', 0, 'n', 0, 'c', 0, 'o', 0, 'd', 0, 'i', 0, 'n', 0, 'g' };
									
	
	public XMLInputStream(InputStream in, Logger logger)
	{
		this.in = in;
		this.count = 0;
		this.buffer = new ByteArrayOutputStream();
		this.logger = logger;
	}
	
	protected static int searchBytes(byte[] haystack, int start, int end, byte[] needle)
	{
		int nl = needle.length;
		int realend = end > haystack.length ? haystack.length : end;
		realend = end - nl;
		int result = -1;
		for (int i = start; i < realend; ++i)
		{
			int j = 0;
			for(; j < nl; ++j)
			{
				if(haystack[i+j] != needle[j])
				{
					break;
				}
			}
			if(j == nl)
			{
				result = i+nl;
			}
		}
		
		return result;
	}
	
	public int consume()
		throws IOException
	{
		byte[] bb = new byte[READ_BUFFER_SIZE];
		while(read(bb) >= 0) {	}
		return getCount();
	}
	
	private static boolean is16BitLE(int offset, byte[] bb)
	{
		boolean result = false;
		if(bb.length >= (offset+2) && bb[offset] == 0 && bb[offset+1] != 0)
		{
			
			result = true;
		}
		return result;
	}
	private static boolean is16BitBE(int offset, byte[] bb)
	{
		boolean result = false;
		if(bb.length >= (offset+2) && bb[offset] != 0 && bb[offset+1] == 0)
		{
			result = true;
		}
		return result;
	}
	
	private static boolean hasUTF32BEMarker(int offset, byte[] bb)
	{
		boolean result = false;
		int len = bb.length;
		if(len >= offset + 4 
			&& (bb[offset] & 0xff) == 0x00 
			&& (bb[offset+1] & 0xff) == 0x00
			&& (bb[offset+2] & 0xff) == 0xfe
			&& (bb[offset+3] & 0xff) == 0xff)
		{
			result = true;
		}
		return result;
	}
	
	private static boolean hasUTF32LEMarker(int offset, byte[] bb)
	{
		boolean result = false;
		int len = bb.length;
		if(len >= offset + 4 
			&& (bb[offset] & 0xff) == 0xff 
			&& (bb[offset+1] & 0xff) == 0xfe
			&& (bb[offset+2] & 0xff) == 0x00
			&& (bb[offset+3] & 0xff) == 0x00)
		{
			result = true;
		}
		return result;
	}

	private static boolean hasUTF8Marker(int offset, byte[] bb)
	{
		boolean result = false;
		int len = bb.length;
		if(len >= offset + 3 
			&& (bb[offset] & 0xff) == 0xef 
			&& (bb[offset+1] & 0xff) == 0xbb
			&& (bb[offset+2] & 0xff) == 0xbf)
		{
			result = true;
		}
		return result;
	}
	
	private static boolean hasUTF16BEMarker(int offset, byte[] bb)
	{
		boolean result = false;
		int len = bb.length;
		if(len >= offset + 2 
			&& (bb[offset] & 0xff) == 0xfe 
			&& (bb[offset+1] & 0xff) == 0xff)
		{
			result = true;
		}
		return result;
	}
	private static boolean hasUTF16LEMarker(int offset, byte[] bb)
	{
		boolean result = false;
		int len = bb.length;
		if(len >= offset + 2 
			&& (bb[offset] & 0xff) == 0xff 
			&& (bb[offset+1] & 0xff) == 0xfe)
		{
			result = true;
		}
		return result;
	}
	public String guessEncoding(int offset)
		throws UnsupportedEncodingException
	{
		String parsedEncoding = null;
		String detectedEncoding = null;
		byte[] bb = getBuffer();
		int len = bb.length;
		int endpoint = MAX_HEADER_LEN > len ? len : MAX_HEADER_LEN;
// detect optional mark characters		
		if(hasUTF32BEMarker(offset,bb))
		{
			detectedEncoding = "UTF-32"; // big-endian
		}
		else if(hasUTF32LEMarker(offset,bb))
		{
			detectedEncoding = "UTF-32LE";
		}
		else if(hasUTF16BEMarker(offset,bb))
		{
			detectedEncoding = "UTF-16"; // big-endian
		}
		else if(hasUTF16LEMarker(offset,bb))
		{
			detectedEncoding = "UTF-16LE";
		}
		else if(hasUTF8Marker(offset,bb))
 		{
			detectedEncoding = "UTF-8";
		}
// look for embedded evidence		
		else if(is16BitBE(offset,bb))
		{
			detectedEncoding = "UTF-16"; // big-endian
		}
		else if(is16BitLE(offset,bb))
		{
			detectedEncoding = "UTF-16LE";
		}
		else if(len > (offset + 4) && bb[offset] == '<' && bb[offset+1] == '?')
		{
			int n = offset + 2;
			for(; n < endpoint; ++n)
			{
				if(bb[n] == '?')
				{
					break;
				}
			}
			
			if(n < endpoint)
			{
//				if(searchBytes(bb,offset,n,ENCODING_8_BIT) != -1)
//				{
//					detectedEncoding = "US-ASCII";
//				}
//				else 
					if(searchBytes(bb,offset,n,ENCODING_16_BIT) != -1)
				{
					detectedEncoding = "UTF-16";
				}
			}
		}
			
//		if(detectedEncoding != null)
		{
			String frag;
			if(detectedEncoding != null)
			{
				frag = new String(bb,offset,endpoint,detectedEncoding);
			}
			else
			{
				frag = new String(bb,offset,endpoint,"UTF-8");
			}
			int encoff = frag.indexOf("encoding");
			if(encoff != -1)
			{
				encoff = frag.indexOf("=",encoff);
				if(encoff != -1)
				{
					frag = frag.substring(encoff + 1).trim();
					char c = frag.charAt(0);
					if(c == '\'')
					{
						encoff = frag.indexOf('\'',1);
						if(encoff != -1)
						{
							parsedEncoding = frag.substring(1,encoff);
						}
					}
					else if(c == '\"')
					{
						encoff = frag.indexOf('\"',1);
						if(encoff != -1)
						{
							parsedEncoding = frag.substring(1,encoff);
							if(detectedEncoding != null && ! parsedEncoding.equals(detectedEncoding))
							{
								logger.warning("Encoding type declared in XML header `" + parsedEncoding 
									+ "' disgrees with detected encoding of `" + detectedEncoding + "'"); 
							}
							try
							{
								if(! Charset.isSupported(parsedEncoding))
								{
									logger.warning("Encoding type declared in XML header `" + parsedEncoding
										+ "' is not support by this JVM");
									parsedEncoding = null;
								}
							}
							catch(IllegalCharsetNameException e)
							{
								logger.warning("Encoding type declared in XML header `" + parsedEncoding
									+ "' is not support by this JVM");
								parsedEncoding = null;
							}
						}
					}
				}
			}
		}
		return parsedEncoding;
	}
	
	public InputStream getInputStream(int start)
	{
		final byte[] bytes = getBuffer();
		return new ByteArrayInputStream(bytes,start, bytes.length - start);
	}
	
	public int read(byte[] bb)
		throws IOException
	{
		return read(bb,0,bb.length);
	}
	
	public int read(byte[] bb, int offset, int len)
		throws IOException
	{
		int result = in.read(bb,offset,len);
		if(result > 0)
		{
			count += result;
			buffer.write(bb,offset,result);
		}
		return result;
	}
	public int read()
		throws IOException
	{
		int c =  in.read();
		if(c != -1)
		{
			++count;
			buffer.write((char) c);			
		}
		return c;
	}
	public int getCount()
	{
		return count;
	}
	public String getText(String encoding)
		throws UnsupportedEncodingException
	{
		byte[] bb = getBuffer();
		int offset = 0;
		if(encoding == null)
		{
			return new String(bb);
		}
		else
		{
			if(encoding.equalsIgnoreCase("UTF-8") && hasUTF8Marker(offset,bb))
			{
				offset = 3;
			}
			else if(encoding.equalsIgnoreCase("UTF-16") && hasUTF16BEMarker(offset,bb))
			{
				offset = 2;
			}
			return new String(bb,offset,bb.length - offset,encoding);
		}
	}
	public byte[] getBuffer()
	{
		if(bytes == null)
		{
			bytes = buffer.toByteArray();
		}
		return bytes;
	}
}