/*
 * This software is copyright (c) 2004 by Shadowplay.net. It is released 
 * under the terms of the Artistic License.  Please refer to the file 
 * LICENSE for details.
 */
package net.shadowplay.wurml.input;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;

import net.shadowplay.wurml.filter.InputFilter;

import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;


/**
 * @author Michael Dykman for Shadowplay.net 
 */
public class InputContainer
{
	Map urlMap;
	public InputContainer()
	{
		urlMap = new TreeMap(new URLComparator());
	}

	public InputContainer(Node node)
	{
		this();
		DocumentImpl impl = new DocumentImpl();
		NodeIterator it = impl.createNodeIterator(node,NodeFilter.FILTER_ACCEPT,
			new URLFilter());
		Node n;
		String siteName;
		Map namedValues;
		List values;
		while((n = it.nextNode()) != null)
		{
			siteName = n.getFirstChild().getNodeValue();
			namedValues = new HashMap();
			values = new ArrayList();
			NodeIterator iit = impl.createNodeIterator(n.getParentNode(),NodeFilter.FILTER_ACCEPT, 
				new InputFilter());
			while((n = iit.nextNode()) != null)
			{
				values = new ArrayList();
				NamedNodeMap attr = n.getAttributes();
				
				String valName = attr.getNamedItem("name").getNodeValue();
				Node vn;
				NodeIterator vit = impl.createNodeIterator(n,NodeFilter.FILTER_ACCEPT, 
					new ValueFilter());
				while((vn = vit.nextNode()) != null)
				{
					String v = vn.getFirstChild().getNodeValue();
					values.add(v);
				}
				namedValues.put(valName,values);
			}
			urlMap.put(siteName,namedValues);
		}
	}
	
	public synchronized List getValues(String url,String inputName)
	{
		List result = null;
		Iterator it = urlMap.keySet().iterator();
		Map m;
		while(result == null && it.hasNext())
		{
			String siteKey = it.next().toString();
			if(url.startsWith(siteKey))
			{
				m = (Map) urlMap.get(siteKey);
				result = (List) m.get(inputName);
			}
		}
		return result == null ? new ArrayList() : result;
	}
	
	static class URLComparator implements Comparator
	{
		public int compare(Object lhs, Object rhs)
		{
			int result;
			String l = lhs.toString();
			String r = rhs.toString();
			
			if(l.length() > r.length())
			{
				result = -1;
			}
			else
			{
				result = l.compareTo(r);
			}
				
			return result;
		}
	}
	static class ValueFilter implements NodeFilter
	{
		public short acceptNode(Node node)
		{
			if(node.getNodeName().equalsIgnoreCase("value"))
			{
				return NodeFilter.FILTER_ACCEPT;
			}
			else
			{
				return NodeFilter.FILTER_SKIP;
			}
		}
	}
	static class URLFilter implements NodeFilter
	{
		public short acceptNode(Node node)
		{
			if(node.getNodeName().equalsIgnoreCase("url"))
			{
				return NodeFilter.FILTER_ACCEPT;
			}
			else
			{
				return NodeFilter.FILTER_SKIP;
			}
		}
	}
}
