package net.shadowplay.xml;

import net.shadowplay.wurml.ErrorCounter;
import net.shadowplay.wurml.WapConstants;

import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.SAXException;


/**
 * @author Michael Dykman
 */
public abstract class DomParserFactory
{
	public static DocumentBuilder createDocumentBuilder(
		String type,boolean validate,
		ErrorCounter counter, String [] ignoreMessages)
		throws SAXException
	{
		DocumentBuilder parser = null;
		ParseErrorHandler eh = new ParseErrorHandler(counter, ignoreMessages);
		if(type.equals(WapConstants.WML_TYPE))
		{
			parser = new WMLParser(validate);
			parser.setParseErrorHandler(eh);
		}
		else if(type.equals(WapConstants.XHTML_TYPE))
		{
			parser = new XHTMLParser(validate);
			parser.setParseErrorHandler(eh);
		}
		else
		{
			throw new SAXException("no parser avalable for type `" + type + "'");
		}
		return parser;
	}
	
	static abstract class DocumentBuilderImpl 
		extends DOMParser 
	 	implements DocumentBuilder
	{
		private ParseErrorHandler eh;
		protected boolean validate;
		
		public DocumentBuilderImpl(boolean validate)
			throws SAXException
		{
			_init(validate);
		}
		public final void _init(boolean validate)
			throws SAXException
		{
			this.validate = validate;
			setFeature(
				"http://xml.org/sax/features/validation",
				this.validate);
			setFeature(
				"http://apache.org/xml/features/nonvalidating/load-dtd-grammar",
				this.validate);
			setFeature(
				"http://apache.org/xml/features/nonvalidating/load-external-dtd",
				this.validate);
		}

		public void setParseErrorHandler(ParseErrorHandler eh)
		{
			this.eh = eh;
			this.setErrorHandler(eh);
		}
		public ParseErrorHandler getParseErrorHandler()
		{
			return eh;
		}
	}
	
	static class WMLParser extends DocumentBuilderImpl
	{
		public WMLParser(boolean validate)
			throws SAXException
		{
			super(validate);
		}
	}
	
	
	static class XHTMLParser  extends DocumentBuilderImpl
	{
		public XHTMLParser(boolean validate)
			throws SAXException
		{
			super(validate);
		}
	}
}
