package net.shadowplay.xml;

import net.shadowplay.wurml.ErrorCounter;
import net.shadowplay.wurml.WurML;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;


/**
 * @author Michael Dykman
 */
public class ParseErrorHandler  implements ErrorHandler
{
	boolean error = false;
	private StringBuffer details = new StringBuffer();
	private String doc;
	private ErrorCounter counter;
	private String [] ignoreMessages;
	
	public ParseErrorHandler(ErrorCounter counter,
		String [] ignoreMessages)
	{
		this.counter = counter;
		this.ignoreMessages = ignoreMessages;
	}
	public void setDoc(String doc)
	{
		this.doc = doc;
		error = false;
		details = new StringBuffer();
	}
	public String getURL()
	{
		return doc;
	}
	public boolean errorFlag()
	{
		return error;
	}
	private void report(String type, SAXParseException e)
	{
		error = true;
		details
		.append(type).append(": ").append(doc).append(' ')
			.append("line ").append(e.getLineNumber()).append(", ")
			.append("column ").append(e.getColumnNumber())
			.append(" - ").append(e.getMessage())
			.append(WurML.EOL);
	}
	public void fatalError(SAXParseException e)
	{
		counter.incrementErrors();
		report("FATAL XML ERROR",e);
	}
	private boolean ignoreMsg(String msg)
	{
		boolean result = false;
		if(msg != null) for(int i = 0; i < ignoreMessages.length; ++i)
		{
			if(msg.indexOf(ignoreMessages[i]) != -1)
			{
				result = true;
				break;
			}
		}
		return result;
	}
	public void error(SAXParseException e)
	{
		if(! ignoreMsg(e.getMessage()))
		{
			counter.incrementWarnings();
			report("xml error", e);
		}
	}
	public void warning(SAXParseException e)
	{
		counter.incrementWarnings();
		report("xml warning", e);
	}
	public String toString()
	{
		return details.toString();
	}
}


